import QtQuick 1.0
import "components"
import "scripts/youtube.js" as YT
import "scripts/createobject.js" as ObjectCreator

Dialog {
    id: root

    property string videoId

    signal playVideo(variant video)

    onVideoIdChanged: YT.getYouTubeVideos("http://gdata.youtube.com/feeds/api/videos/" + videoId + "/related?v=2.1&max-results=20&alt=json")

    titleText: qsTr("Related videos")
    height: appWindow.height - 100
    destroyOnClosing: true
    content: GridView {
        id: videoList

        anchors.fill: parent
        width: root.width - 20
        height: root.height - titleText.height - 40
        cellWidth: Math.floor(width / 4)
        cellHeight: Math.floor((cellWidth * 3) / 4)
        cacheBuffer: 5000
        maximumFlickVelocity: Settings.maximumFlickVelocity
        flickableDirection: Flickable.VerticalFlick
        clip: true
        model: ListModel {
            id: videoListModel

            property bool loading : false
            property bool error: false
            property int totalResults
            property int page : 0
        }
        delegate: YTRelatedVideoDelegate {
            id: delegate

            width: videoList.cellWidth
            height: videoList.cellHeight
            onClicked: {
                root.playVideo([ObjectCreator.cloneObject(videoListModel.get(index))]);
                root.accept();
            }
        }

        ScrollBar {}

        Label {
            id: noResultsText

            anchors.centerIn: videoList
            font.pixelSize: _LARGE_FONT_SIZE
            font.bold: true
            color: "#4d4d4d"
            horizontalAlignment: Text.AlignHCenter
            verticalAlignment: Text.AlignVCenter
            text: qsTr("No related videos")
            visible: (!videoListModel.loading) && (videoListModel.count == 0)
        }

        BusyDialog {
            anchors.centerIn: parent
            visible: videoListModel.loading
        }
    }
}
